package ca.canadapost.cpcdp.postoffice;

import ca.canadapost.cpcdp.postoffice.generated.messages.Messages;
import ca.canadapost.cpcdp.postoffice.generated.postoffice.*;

import com.sun.jersey.api.client.*;
import com.sun.jersey.api.client.config.*;

import javax.xml.bind.JAXBContext;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Properties;

/**
 * Sample code for the GetNearestPostOffice Canada Post service.
 * <p>
 * The GetNearestPostOffice service returns a list of Post Offices 
 * near a given location including information such as address and phone number.
 * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 * <p> 
 *  Jersey REST client using JAXB for GetNearestPostOffice Request.  <br>
 *  USAGE:<pre>
 *        GetNearestPostOfficeClient client = new GetNearestPostOfficeClient(username, password);
 *        ClientResponse response = client.getNearestPostOffice();
 *        // do whatever with response
 *        client.close();
 *  </pre>
 *
 */
public class GetNearestPostOfficeClient {

    private Client aClient;
    
    // REST URI
    private static final String LINK = "https://ct.soa-gw.canadapost.ca/rs/postoffice?postalCode=K2B8J6&maximum=4";
    
    // Main class for local testing only
    public static void main(String[] args) {

    	// Your username and password are imported from the following file    	
    	// CPCWS_PostOffice_Java_Samples/user.properties 
    	Properties userProps = new Properties();
    	FileInputStream propInputStream;
		try {
			propInputStream = new FileInputStream("user.properties");
			userProps.load(propInputStream);
			propInputStream.close(); // better in finally block
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace(System.out);
			return;
		}
    	String username = userProps.getProperty("username");
    	String password = userProps.getProperty("password");
						
		// Execute GetNearestPostOffice Request
        GetNearestPostOfficeClient myClient = new GetNearestPostOfficeClient(username, password);
        ClientResponse resp = myClient.getNearestPostOffice();
        InputStream respIS = resp.getEntityInputStream();
        
        System.out.println("HTTP Response Status: " + resp.getStatus() + " " + resp.getClientResponseStatus());

        // Example of using JAXB to parse xml response
        JAXBContext jc;
        try {
			jc = JAXBContext.newInstance(PostOfficeList.class, Messages.class);
            Object entity = jc.createUnmarshaller().unmarshal(respIS);
            // Determine whether response data matches PostOfficeList schema.
            if (entity instanceof PostOfficeList) {            	
            	PostOfficeList postOfficeList = (PostOfficeList) entity;    
            
                for (Iterator<PostOfficeList.PostOffice> iter = postOfficeList.getPostOffices().iterator(); iter.hasNext();) {
                	PostOfficeList.PostOffice postOffice = (PostOfficeList.PostOffice) iter.next();
                	System.out.println("Office Name: " + postOffice.getName());
	                System.out.println("Office Id: " + postOffice.getOfficeId());
	                System.out.println("Address: " + postOffice.getAddress().getOfficeAddress());
	                System.out.println();
                }
            } else {
                // Assume Error Schema
                Messages messageData = (Messages) entity;
                for (Iterator<Messages.Message> iter = messageData.getMessage().iterator(); iter.hasNext();) {
                    Messages.Message aMessage = (Messages.Message) iter.next();
                    System.out.println("Error Code: " + aMessage.getCode());
                    System.out.println("Error Msg: " + aMessage.getDescription());
                }
            }
        } catch (Exception e) {
        	e.printStackTrace(System.out);
        }

        myClient.close();
    }

    public GetNearestPostOfficeClient(String username, String password) {
        ClientConfig config = new DefaultClientConfig();
        aClient = Client.create(config);
        aClient.addFilter(new com.sun.jersey.api.client.filter.HTTPBasicAuthFilter(username, password));
    }

    public ClientResponse getNearestPostOffice() throws UniformInterfaceException {    	
    	WebResource aWebResource = aClient.resource(LINK);
        return aWebResource.accept("application/vnd.cpc.postoffice+xml").acceptLanguage("en-CA").get(ClientResponse.class);
    }

    public void close() {
        aClient.destroy();
    }
}
